@@ -30,10 +30,13 @@ from gpodder import libtagupdate
 
 from gpodder.liblogger import log
 from gpodder.libgpodder import gl
+from gpodder.dbsqlite import db
 
+gpod_available = True
 try:
     import gpod
 except:
+    gpod_available = False
     log('(gpodder.sync) Could not find gpod')
 
 try:
@@ -133,8 +136,9 @@ class Device(services.ObservableService):
     def __init__(self):
         self.cancelled = False
         self.allowed_types = ['audio', 'video']
-
-        signals = ['progress', 'sub-progress', 'status', 'done']
+        self.errors = []
+        self.tracks_list = []
+        signals = ['progress', 'sub-progress', 'status', 'done', 'post-done']
         services.ObservableService.__init__(self, signals)
 
     def open(self):
@@ -146,8 +150,9 @@ class Device(services.ObservableService):
 
     def close(self):
         self.notify('status', _('Writing data to disk'))
-        os.system('sync')
+        successful_sync = not os.system('sync')
         self.notify('done')
+        self.notify('post-done', self, successful_sync)
         return True
 
     def add_tracks(self, tracklist=[], force_played=False):
@@ -157,10 +162,10 @@ class Device(services.ObservableService):
 
             self.notify('progress', id+1, len(tracklist))
 
-            if not track.is_downloaded():
+            if not track.was_downloaded(and_exists=True):
                 continue
 
-            if track.is_played() and gl.config.only_sync_not_played and not force_played:
+            if track.is_played and gl.config.only_sync_not_played and not force_played:
                 continue
 
             if track.file_type() not in self.allowed_types:
@@ -170,7 +175,7 @@ class Device(services.ObservableService):
 
             if gl.config.on_sync_mark_played:
                 log('Marking as played on transfer: %s', track.url, sender=self)
-                gl.history_mark_played(track.url)
+                db.mark_episode(track.url, is_played=True)
 
             if added and gl.config.on_sync_delete:
                 log('Removing episode after transfer: %s', track.url, sender=self)
@@ -194,6 +199,17 @@ class Device(services.ObservableService):
     def remove_track(self, track):
         pass
 
+    def get_free_space(self):
+        pass
+
+    def episode_on_device(self, episode):
+        return self._track_on_device(episode)
+
+    def _track_on_device( self, track_name ):
+        for t in self.tracks_list:
+            if track_name == t.title:
+                return t
+        return False
 
 class iPodDevice(Device):
     def __init__(self):
@@ -203,6 +219,12 @@ class iPodDevice(Device):
 
         self.itdb = None
         self.podcast_playlist = None
+        
+
+    def get_free_space(self):
+        # Reserve 10 MiB for iTunesDB writing (to be on the safe side)
+        RESERVED_FOR_ITDB = 1024*1024*10
+        return util.get_free_disk_space(self.mountpoint) - RESERVED_FOR_ITDB
 
     def open(self):
         if not os.path.isdir(self.mountpoint):
@@ -218,6 +240,10 @@ class iPodDevice(Device):
 
         if self.podcasts_playlist:
             self.notify('status', _('iPod opened'))
+
+            # build the initial tracks_list
+            self.tracks_list = self.get_all_tracks()
+
             return True
         else:
             return False
@@ -263,7 +289,7 @@ class iPodDevice(Device):
 
             t = SyncTrack(track.title, length, modified, libgpodtrack=track, playcount=track.playcount, released=released, podcast=track.artist)
             tracks.append(t)
-        return tracks
+        return tracks        
 
     def remove_track(self, track):
         self.notify('status', _('Removing %s') % track.title)
@@ -283,7 +309,7 @@ class iPodDevice(Device):
         for track in gpod.sw_get_playlist_tracks(self.podcasts_playlist):
             if episode.url == track.podcasturl:
                 if track.playcount > 0:
-                    gl.history_mark_played(track.podcasturl)
+                    db.mark_episode(track.podcasturl, is_played=True)
                 # Mark as played on iPod if played locally (and set podcast flags)
                 self.set_podcast_flags(track)
                 return True
@@ -291,13 +317,8 @@ class iPodDevice(Device):
         original_filename = str(episode.local_filename())
         local_filename = original_filename
 
-        # Reserve 10 MiB for iTunesDB writing (to be on the safe side)
-        RESERVED_FOR_ITDB = 1024*1024*10
-        space_for_track = util.get_free_disk_space(self.mountpoint) - RESERVED_FOR_ITDB
-        needed = util.calculate_size(local_filename)
-
-        if needed > space_for_track:
-            log('Not enough space on %s: %s available, but need at least %s', self.mountpoint, util.format_filesize(space_for_track), util.format_filesize(needed), sender = self)
+        if util.calculate_size(original_filename) > self.get_free_space():
+            log('Not enough space on %s, sync aborted...', self.mountpoint, sender = self)
             self.errors.append( _('Error copying %s: Not enough free disk space on %s') % (episode.title, self.mountpoint))
             self.cancelled = True
             return False
@@ -370,7 +391,8 @@ class iPodDevice(Device):
     def set_podcast_flags(self, track):
         try:
             # Set blue bullet for unplayed tracks on 5G iPods
-            if gl.history_is_played(track.podcasturl):
+            episode = db.load_episode(track.podcasturl)
+            if episode['is_played']:
                 track.mark_unplayed = 0x01
                 if track.playcount == 0:
                     track.playcount = 1
@@ -434,10 +456,15 @@ class MP3PlayerDevice(Device):
         self.buffer_size = 1024*1024 # 1 MiB
         self.scrobbler_log = []
 
+    def get_free_space(self):
+        return util.get_free_disk_space(self.destination)
+
     def open(self):
         self.notify('status', _('Opening MP3 player'))
         if util.directory_is_writable(self.destination):
             self.notify('status', _('MP3 player opened'))
+            # build the initial tracks_list
+            self.tracks_list = self.get_all_tracks()
             if gl.config.mp3_player_use_scrobbler_log:
                 mp3_player_mount_point = util.find_mount_point(self.destination)
                 # If a moint point cannot be found look inside self.destination for scrobbler_log_filenames
@@ -463,7 +490,7 @@ class MP3PlayerDevice(Device):
         else:
             folder = self.destination
 
-        from_file = episode.local_filename()
+        from_file = util.sanitize_encoding(episode.local_filename())
         filename_base = util.sanitize_filename(episode.sync_filename(), self.MAX_FILENAME_LENGTH)
 
         to_file = filename_base + os.path.splitext(from_file)[1].lower()
@@ -483,10 +510,10 @@ class MP3PlayerDevice(Device):
                 log('Cannot create folder on MP3 player: %s', folder, sender=self)
                 return False
 
-        if (gl.config.mp3_player_use_scrobbler_log and not episode.is_played()
+        if (gl.config.mp3_player_use_scrobbler_log and not episode.is_played
                 and [episode.channel.title, episode.title] in self.scrobbler_log):
             log('Marking "%s" from "%s" as played', episode.title, episode.channel.title, sender=self)
-            gl.history_mark_played(episode.url)
+            db.mark_episode(episode.url, is_played=True)
 
         if gl.config.rockbox_copy_coverart and not os.path.exists(os.path.join(folder, 'cover.bmp')):
             log('Creating Rockbox album art for "%s"', episode.channel.title, sender=self)
@@ -567,6 +594,10 @@ class MP3PlayerDevice(Device):
             tracks.append(t)
         return tracks
 
+    def episode_on_device(self, episode):
+        e = util.sanitize_filename(episode.sync_filename(), gl.config.mp3_player_max_filename_length)
+        return self._track_on_device(e)
+
     def remove_track(self, track):
         self.notify('status', _('Removing %s') % track.title)
         util.delete_file(track.filename)
